package loanrepayments;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Title:        Calculating the Interest Paid on a Loan
 * Description:  People are often thankfully unaware of their extra costs of
 * interest on the loan.  This little program will calculate the incremental
 * costs, after checking the ranges and types of data entered.
 * The resulting payment data over a period of time can be printed out.
 * Copyright:    Copyright (c) 2003
 * Company:      Bunga Publications
 * @author Rob Ward
 * @version 1.0
 */

public class Frame1 extends JFrame {
  //Add the GUI components to the GUI, NB sizing and placement occur later
  private JPanel contentPane;
  private JLabel jLabel1 = new JLabel();
  private JLabel jLabel2 = new JLabel();
  private JLabel jLabel3 = new JLabel();
  private JLabel jLabel4 = new JLabel();
  private JLabel jLabel5 = new JLabel();
  private JLabel jLabel8 = new JLabel();
  private JTextField amountTxt = new JTextField();
  private JTextField depositTxt = new JTextField();
  private JTextField ratepaTxt = new JTextField();
  private JTextField repaymentsTxt = new JTextField();
  private JLabel monthsToGoLbl = new JLabel();
  private JLabel repaymentsLbl = new JLabel();
  private JButton calculateBtn = new JButton();
  private JButton quitBtn = new JButton();
  private JLabel loanLbl = new JLabel();

  //Construct the frame
  public Frame1() {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  //Component initialization
  private void jbInit() throws Exception  {
    //Note place these lines first or Java will NOT make Frame right size
    this.setSize(new Dimension(440, 320));
    this.setTitle("VITTA 2003 Java Loan Repayments");
    //Now grab the Content Pane and it will resize properly.
    contentPane = (JPanel) this.getContentPane();
    contentPane.setPreferredSize(new Dimension(440, 320));
    contentPane.setLayout(null);
    //create a Project icon, place graphic in the Class Directory
    setIconImage(Toolkit.getDefaultToolkit().createImage(
                                Frame1.class.getResource("interest.gif")));
    //OK, Now start sizing and laying out the graphics

    //Assorted labels for user friendly GUI
    jLabel8.setText("Loan Amount");
    jLabel8.setBounds(new Rectangle(26, 121, 131, 24));
    loanLbl.setText("How Much Borrowed");
    loanLbl.setBounds(new Rectangle(169, 121, 128, 24));
    jLabel1.setFont(new java.awt.Font("Monospaced", 1, 18));
    jLabel1.setText("Loans Repayment Calculator");
    jLabel1.setBounds(new Rectangle(55, 8, 293, 30));
    jLabel2.setText("Total Amount");
    jLabel2.setBounds(new Rectangle(26, 50, 131, 24));
    jLabel3.setText("Deposit");
    jLabel3.setBounds(new Rectangle(26, 86, 131, 24));
    jLabel4.setText("Interest rate PA");
    jLabel4.setBounds(new Rectangle(26, 157, 131, 24));
    jLabel5.setText("Rate Paid Back Monthly");
    jLabel5.setBounds(new Rectangle(26, 192, 131, 24));

    //Labels used for showing output on the GUI
    monthsToGoLbl.setText("Months to Pay Back");
    monthsToGoLbl.setBounds(new Rectangle(169, 228, 128, 24));
    repaymentsLbl.setText("Repayments");
    repaymentsLbl.setBounds(new Rectangle(26, 263, 269, 24));

    //Texfield for the total amount in the loan
    amountTxt.setToolTipText("");
    amountTxt.setText("10000");
    amountTxt.setHorizontalAlignment(SwingConstants.RIGHT);
    amountTxt.setBounds(new Rectangle(169, 50, 128, 24));

    //Texfield for the deposit
    depositTxt.setText("1000");
    depositTxt.setHorizontalAlignment(SwingConstants.RIGHT);
    depositTxt.setBounds(new Rectangle(169, 86, 128, 24));

    //Texfield for the interest rate
    ratepaTxt.setText("6");
    ratepaTxt.setHorizontalAlignment(SwingConstants.RIGHT);
    ratepaTxt.setBounds(new Rectangle(169, 157, 128, 24));

    //Texfield for the repayments
    repaymentsTxt.setText("300");
    repaymentsTxt.setHorizontalAlignment(SwingConstants.RIGHT);
    repaymentsTxt.setBounds(new Rectangle(169, 192, 128, 24));

    //Button on GUI to initiate calculation of repayments
    calculateBtn.setBounds(new Rectangle(304, 194, 90, 42));
    calculateBtn.setHorizontalAlignment(SwingConstants.LEFT);
    calculateBtn.setText("Calculate");
    calculateBtn.setIcon(new ImageIcon( Frame1.class.getResource( "enter.gif")));
    calculateBtn.setMargin(new Insets(0, 0, 0, 0));
    calculateBtn.setPressedIcon(new ImageIcon( Frame1.class.getResource( "blank.gif")));
    calculateBtn.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        calculateBtn_actionPerformed(e);
      }
    });

    //A quit button to exit the program
    quitBtn.setBounds(new Rectangle(305, 242, 90, 42));
    quitBtn.setHorizontalAlignment(SwingConstants.LEFT);
    quitBtn.setText("QUIT");
    quitBtn.setIcon(new ImageIcon( Frame1.class.getResource( "enter.gif")));
    quitBtn.setMargin(new Insets(0, 0, 0, 0));
    quitBtn.setPressedIcon(new ImageIcon( Frame1.class.getResource( "blank.gif")));
    //Now add the listener that will transfer the processing to further down
    quitBtn.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        quitBtn_actionPerformed(e);
      }
    });

    //Now add the components into the GUI content pane
    contentPane.add(jLabel1, null);
    contentPane.add(jLabel2, null);
    contentPane.add(jLabel3, null);
    contentPane.add(jLabel4, null);
    contentPane.add(jLabel5, null);
    contentPane.add(jLabel8, null);
    contentPane.add(repaymentsLbl, null);
    contentPane.add(loanLbl, null);
    contentPane.add(monthsToGoLbl, null);
    contentPane.add(ratepaTxt, null);
    contentPane.add(amountTxt, null);
    contentPane.add(depositTxt, null);
    contentPane.add(repaymentsTxt, null);
    contentPane.add(calculateBtn, null);
    contentPane.add(quitBtn, null);
  }
  //Overridden so we can exit when window is closed
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      System.exit(0);
    }
  }

  void quitBtn_actionPerformed(ActionEvent e) {
     //Exit the whole application
     System.exit(0);
  }

  void calculateBtn_actionPerformed(ActionEvent e) {
    //Set up some local double precision numbers for the program to work with
    double total=0;
    double deposit=0;
    double loan=0;
    double interest=0;
    double repayment=0;
    double totalrepay=0;
    double months=0;

    //Now do a format and rang check using the methods further down
    if (DoubleRangeCheck(this,amountTxt.getText(),
         "Loan Total",1000, 1000000)){
        total = Double.parseDouble(amountTxt.getText());
        }
    //Assign the number converted from the text in the textfield
    deposit = Double.parseDouble(depositTxt.getText());
    //Assign the loan value the difference between total and deposit
    loan = total-deposit;
    //Display the result on the GUI output label
    loanLbl.setText("$"+loan);
    //Gather the rest of the numbers to so the calculations
    interest = Double.parseDouble(ratepaTxt.getText());
    repayment = Double.parseDouble(repaymentsTxt.getText());
     //Check the repayments are high enough before using the 'while'
    //or the loand and the program will never end !!!
    if (repayment > loan*interest/12/100){
      while(loan >0){
        //repeat the calculation unti the money owed is >0
        loan = loan*((100+(interest/12))/100);
        loan = loan - repayment;
        totalrepay = totalrepay + (loan*interest/12/100);
        //Increment the months variable by 1
        months++;
        }
        //Use a label to output to the GUI
        monthsToGoLbl.setText("Months: "+months);
        //Use a Formatting Class to define the type of formatting
        java.text.DecimalFormat df2p = new java.text.DecimalFormat("###,##0.00");
        //Now apply the formatting class to the number to make a String
        String totalrepaystring = df2p.format(totalrepay);
        repaymentsLbl.setText("Total interest paid is $"+totalrepaystring);
      }
      else{
        //Alert the user with a pop up window message
        JOptionPane.showMessageDialog(this,
            "Please Increase Repayments","Repayment Error" ,2);
        //And add a message to the GUI using a label
        repaymentsTxt.setText("Repayments are not high enough");
      }

  }

    //Following DoubleFormat methods adapted from Programmers Toolkit
    //Rob Ward and David Dawson (C) 2003
    //Available from VITTA at http://www.vitta.org.au
    //
    //Check the format of the string as a double number
    boolean DoubleFormatCheck(Frame parentFrame,String sourceNos,
                                           String prompt){
        double tempNos;
        try{
          tempNos = Double.parseDouble(sourceNos);
          }
        catch(NumberFormatException nosexcep){
          JOptionPane.showMessageDialog(parentFrame,"Incorrect number format",
                     prompt ,2);
          return false;
          }
        return true;
        }
    //DoubleRange
    //Check the range of the double number
    boolean DoubleRangeCheck(Frame parentFrame,String sourceNos,
             String prompt,double minNos, double maxNos){
      double tempNos;
      if(DoubleFormatCheck(parentFrame,sourceNos,prompt)){
        tempNos = Double.parseDouble(sourceNos);
        if(tempNos > maxNos){
          JOptionPane.showMessageDialog(parentFrame,
                 "Number too high", prompt ,2);
          return false;
          }
        if(tempNos < minNos){
          JOptionPane.showMessageDialog(parentFrame,
                  "Number too low", prompt ,2);
          return false;
          }
        }
        else{
          return false;
          }
      return true;
      }
}